package com.sky.ioc.service.device.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.sky.ioc.entity.domain.device.DeviceType;
import com.sky.ioc.entity.domain.device.SecurityDevice;
import com.sky.ioc.mapper.device.DeviceTypeMapper;
import com.sky.ioc.mapper.device.SecurityDeviceMapper;
import com.sky.ioc.service.device.DeviceTypeService;
import com.sky.ioc.tool.ReturnMsg;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class DeviceTypeServiceImpl implements DeviceTypeService {

    @Autowired
    DeviceTypeMapper deviceTypeMapper;

    @Autowired
    SecurityDeviceMapper securityDeviceMapper;

    @Override
    public ReturnMsg batchDel(String device_ids, String device_type_id) {
        if (StringUtils.isNotBlank(device_ids)&&StringUtils.isNotBlank(device_type_id)){
            DeviceType deviceType = deviceTypeMapper.selectById(device_type_id);
            if(deviceType!=null){
                String res = deviceType.getDevice_ids();
                JSONArray ids = JSON.parseArray(device_ids);
                JSONArray array = JSON.parseArray(res);
                array.removeAll(ids);
                deviceTypeMapper.updateById(deviceType);
            }
            return ReturnMsg.ok();
        }else{
            return ReturnMsg.fail("设备id和设备类别不能为空");
        }
    }

    @Override
    public ReturnMsg getListByType(String id, Integer page, Integer page_size) {
        List<DeviceType> typeList = deviceTypeMapper.selectList(new LambdaQueryWrapper<DeviceType>().eq(DeviceType::getId,id));
        List<String> ids = typeList.stream().map(DeviceType::getDevice_ids).collect(Collectors.toList());
        List<SecurityDevice> list = securityDeviceMapper.selectList(new LambdaQueryWrapper<SecurityDevice>().in(SecurityDevice::getDevice_id,ids));
        PageHelper.startPage(page,page_size);
        ReturnMsg<Object> ok = ReturnMsg.ok();
        PageInfo<SecurityDevice> pageInfo = PageInfo.of(list);
        ok.setData(list);
        ok.setTotal(pageInfo.getTotal());
        return ok;
    }

    @Override
    public ReturnMsg editDeviceType(String id, String name) {
        if(StringUtils.isNotBlank(id)&&StringUtils.isNotBlank(name)){
            UpdateWrapper updateWrapper = new UpdateWrapper();
            updateWrapper.eq("id",id);
            updateWrapper.set("name",name);
            deviceTypeMapper.update(null,updateWrapper);
            return ReturnMsg.ok();
        }else{
            return ReturnMsg.fail("设备类别或设备类别id不能为空");
        }

    }

    @Override
    public ReturnMsg delDeviceType(String id) {
        if(StringUtils.isNotBlank(id)){
            deviceTypeMapper.deleteById(id);
            return ReturnMsg.ok();
        }else {
            return ReturnMsg.fail("设备类别id不能为空");
        }
    }

    @Override
    public ReturnMsg addDeviceType(String name) {
        if(StringUtils.isNotBlank(name)){
            DeviceType deviceType = new DeviceType();
            deviceType.setDevice_ids("[]");
            deviceType.setType_name(name);
            deviceTypeMapper.insert(deviceType);
            return ReturnMsg.ok();
        }else{
            return ReturnMsg.fail("设备类别不能为空");
        }
    }

    @Override
    public ReturnMsg getDeviceType(String name) {
        if(StringUtils.isNotBlank(name)){
            name ="%"+name+"%";
            List<DeviceType> lists =deviceTypeMapper.selectList(new LambdaQueryWrapper<DeviceType>().like(DeviceType::getType_name,name));
            return ReturnMsg.ok(lists);
        }else{
            List<DeviceType> lists =deviceTypeMapper.selectList(null);
            return ReturnMsg.ok(lists);
        }

    }

    @Override
    public ReturnMsg importDeviceType(MultipartFile file,String group_id) throws IOException {
        String fileName = file.getOriginalFilename();//文件名称
        String suffixName = fileName.substring(fileName.lastIndexOf("."));//文件后缀名
        //创建上传的工作表
        InputStream inputStream = file.getInputStream();
        Workbook workbook = null;
        if(suffixName.equals(".xls")){
            workbook = new HSSFWorkbook(inputStream);
        }else if(suffixName.equals(".xlsx")){
            workbook = new XSSFWorkbook(inputStream);
        }else{
            return ReturnMsg.fail("文件格式不正确");
        }
        if(workbook!=null){
            //读取数据
            Sheet sheet = null; //表
            Row row = null;//行
            Cell cell = null;//单元格
            List<Map<String,Object>> data = new ArrayList<>();//存放读取的内容
            int number = workbook.getNumberOfSheets();
            for (int i = 0; i < number ; i++) { //遍历sheets
                sheet = workbook.getSheetAt(i);
                int rownum = sheet.getLastRowNum()+1;
                for (int j = 1; j < rownum; j++) {//遍历sheet的row 忽略第一行表头
                    //表格内容
                    row = sheet.getRow(j);
                    if (row==null){
                        continue;
                    }
                    String id = row.getCell(0).toString();
                    String device_name = row.getCell(1).toString();
                    String device_id = row.getCell(2).toString();
                    double device_type = row.getCell(3).getNumericCellValue();
                    String device_location = row.getCell(4).toString();
                    String loop_detail = row.getCell(5).toString();
                    String gateway_code = row.getCell(6).toString();
                    String gateway_ip = row.getCell(7).toString();
                    String usage = row.getCell(8).toString();
                    double status = row.getCell(9).getNumericCellValue();
                    SecurityDevice securityDevice = new SecurityDevice();
                    securityDevice.setDevice_id(device_id);
                    securityDevice.setDevice_name(device_name);
                    securityDevice.setDevice_type((int) device_type);
                    securityDevice.setDevice_location(device_location);
                    securityDevice.setLoop_detail(loop_detail);
                    securityDevice.setGateway_code(gateway_code);
                    securityDevice.setGateway_ip(gateway_ip);
                    securityDevice.setUsage(usage);
                    securityDevice.setStatus((int) status);
                    securityDeviceMapper.insert(securityDevice);
                }

            }

            return  ReturnMsg.ok(data);
        }else{
            return ReturnMsg.fail("Excel工作簿为空");
        }

    }


}
